#!/usr/local/ActivePerl-5.14/bin/perl -w
use feature 'unicode_strings';
use utf8;
use warnings;
use strict;
use File::Basename;
use feature qw(switch say);

my %chartomyth;
my $chartomyth;
my %mythtochar;
my $mythtochar;
my $buffer;
my $rv;
my $tagFile;
my ( $charmapFile, $dirName ) = @ARGV;


#basic error checking
my $countArgs = @ARGV;
if ($countArgs != 2) {
die (
     "Error:\tYou did not provide enough arguments.\n\n" .
     "Syntax: perl cleanmyth2unicode.pl CHARMAPFILENAME \"TAGDIRECTORY\"\n\n".
     "\tCHARMAPFILENAME = the name of your character map file\n".
     "\t\"TAGDIRECTORY\" Valid Choices (Must be in quotes):\n" .
     "\t\t\"dialog string lists\"\n\t\t\"string lists\"\n\t\t\"text\"\n\t\t\"templates\"\n"


    );
}
if ( !$dirName ) { die("Fatal Error:\tYou forgot to specify a tag directory.\n"); }

given ($dirName) {
    when ("dialog string lists") { break; }
    when ("string lists")        { break; }
    when ("text")                { break; }
    when ("templates")           { break; }
    default {
        die("Fatal Error:\tInvalid Tag Type\n\nValid Choices:\n\t\"dialog string lists\"\n\t\"string lists\"\n\t\"text\"\n\t\"templates\"\n"
        );
    }
}


mkdir("$dirName-headers");
mkdir("$dirName-unicode");


# Create the first 32 characters that are reserved by Myth and add to the $chartomyth and $mythtochar hashes
for ( 0 ... 31 ) {

    #print "Hex:\t".sprintf("%x",$_)."\tMyth Number:$_"."\n";
    #print "Hex:\t".pack("H*",sprintf("%x",$_))."\tMyth Number:$_"."\n";

    $chartomyth{ pack( "H*", sprintf( "%x", $_ ) ) } = $_;
    $mythtochar{$_} = sprintf( "%x", $_ );
}


# Open the $charmapFile from command line arguments and assign each character and their slot to $chartomyth and $mythtochar
open CHARMAP, "<:encoding(UTF16-LE)", "$charmapFile";

my $list = <CHARMAP>;
my $i    = 32;
for my $ch ( unpack( '(a)*', $list ) ) {
    binmode STDOUT, ":utf8";


    # chartomyth [character] = slot
    $chartomyth{$ch} = $i;

    # mythtochar [ slot ] = character
    $mythtochar{$i}  = $ch;

    # Uncomment the next line to print the charmap values
    #printf "Character:\t%s\t%2.2x\t%s\n", $ch, ord($ch),$i;
    $i++;
}

# Open the given directory $dirName from the command line arguments so each file may be read
opendir( TAGSDIRECTORY, $dirName ) or die "can't opendir $dirName: $!";

# while there's files in TAGSDIRECTORY...
while ( defined( $tagFile = readdir(TAGSDIRECTORY) ) ) {

    # Skip pesky UNIX/Mac files
    next if $tagFile =~ /^\.\.?$/;    # skip . and .. # ...
    next if $tagFile =~ /DS_Store/;
    
    # Get just the name of the tag file without any extension or the directory it's under
    my $name = basename($tagFile);


    print "Converting \"" . $name . "\" to a unicode text file...\n";

    binmode STDOUT, ":utf8";

    # Open the tag file for the purpose of getting its first 64 bytes
    open TAGFILE, "$dirName" . "/" . "$tagFile"
      or die("Fatal Error:\tFailed to read data file for header information:\t[$tagFile]\n$!");

    # I wonder what would happen if this line were removed?
    binmode TAGFILE;

    # Get the size of the tag file.. this is important for reading later on
    my $size = -s "$dirName/$tagFile";

    # How many bytes do we need to read after the first 64 bytes of the header?
    my $headlessSize = $size - 64;

    # read the first 64 bytes
    $rv = read( TAGFILE, $buffer, 64 )
      or die("Fatal Error:\tFailed to read 64 bytes to buffer for TAGFILE:\t[$tagFile]\n");

    # assign what the previous line read in to $header (should be the first 64 bytes)
    my $header = unpack( "H*", $buffer );

    
    # open HEADERFILE > so we can write the $header to it
    open HEADERFILE, ">:raw", "$dirName" . '-headers/' . "$name.header"
      or die "Fatal Error:\tCould not create file: [$name.header]\n$!";

    # print the $header to HEADERFILE
    print HEADERFILE pack( "H*", $header );

    # Read the rest of TAGFILE's bytes, using $headlessSize created earlier to determine how much to read
	# If you experience an error at line 129 it means the tag file wasn't 64 bytes.
    $rv = read( TAGFILE, $buffer, $headlessSize )
      or die "Fatal Error:\tCouldn't read beyond 64 bytes from TAGFILE:\t[$tagFile]\n";

    # Unpack all the bytes to $lines in hex form
    my $lines    = unpack( "H*",    $buffer );

    # Group the bytes of $lines into an array
    my @fulltext = unpack( "(A2)*", $lines );

    binmode STDOUT, ":utf8";

    # open UNICODE $tagFile.txt to write the text to
    open UNICODEFILE, ">:raw:encoding(UTF16-LE):utf8",
      "$dirName" . '-unicode/' . "$tagFile.txt"
      or die("Could not open unicode text file. $!\n");
    
     # Print FEFF to beginning of the file so it's identified as UTF-16LE
     print UNICODEFILE "\x{FEFF}";

     # Go through the array, 1 byte at a time and print it to the UNICODEFILE text file
     foreach (@fulltext) {

	# If the byte ne 0D we print the appropriate charmapped values to the UNICODEFILE text file
	# If the byte eq 0D we print the proper newline characters 000D and 000A.
        if ( $_ ne "0d" ) {

	    # I can't remember why I had to do it this way (hex < 32) , but it works for the first 32 characters.
	    # The first 32 characters Myth reserved were printing incorrectly otherwise.
	    # So if the character's hex is over 32 it will print with the pack unpack mythtochar line

            if ( hex($_) < 32 ) {
                print UNICODEFILE pack( "H*", $_ );
            }
            else {
                print UNICODEFILE pack( "U*", unpack( "U", $mythtochar{ hex($_) } ) );
            }
        }
        elsif ( $_ eq "0d" ) {
			#0D needs to be converted to 0D000A00. Guess it's a Unicode thing.
            print UNICODEFILE "\x{000D}\x{000A}";
        }
    }
}   