#!/usr/local/ActivePerl-5.14/bin/perl -w
use feature 'unicode_strings';
use utf8;
use warnings;
use strict;
use Data::Dumper;
use File::Basename;
use feature qw(switch say);

my %charmap = ();
my $charmap;
my ( $charfile, $tagType ) = @ARGV;
my $tagLine;
my $tagSize;
my $tagrv;
my $tagHeader;
my $tagbuffer;
my $tagTextFile;

#basic error checking
my $countArgs = @ARGV;
if ($countArgs != 2) {
die (
     "Error:\tYou did not provide enough arguments.\n\n" .
     "Syntax: perl cleanuni2myth.pl CHARMAPFILENAME \"TAGTYPE\"\n\n".
     "\tCHARMAPFILENAME = the name of your character map file\n".
     "\t\"TAGTYPE\" Valid Choices (Must be in quotes):\n" .
     "\t\t\"dialog string lists\"\n\t\t\"string lists\"\n\t\t\"text\"\n\t\t\"templates\"\n"


    );
}
if ( !$tagType ) { die("Error: You forgot to specify a tag directory.\n"); }

given ($tagType) {
    when ("dialog string lists") { break; }
    when ("string lists")        { break; }
    when ("text")                { break; }
    when ("templates")           { break; }
    default {
        die("Fatal Error:\tInvalid Tag Type\n\nValid Choices:\n\t\"dialog string lists\"\n\t\"string lists\"\n\t\"text\"\n\t\"templates\"\n"
        );
    }
}


# Create string that names the folder will contain the tags we're creating
# Then create the folder with said name. If the folder doesn't exist it will be created.
# If the folder already exists, that's okay too.
# A more industrious person might think about popping up an error if the folder's already there.
# You would hate to overwrite something, right?

my $newTagsFolder = $tagType . "-unicode_tags";
mkdir("$newTagsFolder");

# Create the string that represents the folder name of where our unicode text files are stored
my $uniDirName = $tagType . "-unicode";

for ( 0 ... 31 ) {

   # Uncomment the following line to see what the charmap characters will be translating to
   #print "Hex:\t" . pack( "H*", sprintf( "%x", $_ ) ) . "\tMyth Number:$_" . "\tOrdi:" . ord( pack( "H*", sprintf( "%x", $_ ) ) ) . "\n";

    $charmap{ pack( "H*", sprintf( "%x", $_ ) ) } = $_;
}

# Open the directory of unicode text files specified by $uniDirName to get a list of all the files needing converted
opendir( UNICODETEXTFILES, $uniDirName ) or die "Error: Can't opendir $uniDirName.\n$!";
print "\n";
print 'The $tagType is: ' . $tagType . "\n";
print 'The $uniDirName is: ' . $uniDirName . "\n";
print "\n";
# Loop through each file name and act upon it
while ( defined( $tagTextFile = readdir(UNICODETEXTFILES) ) ) {

    # Ignore those pesky Mac/Unix files
    next if $tagTextFile =~ /^\.\.?$/;    # skip . and .. # ...
    next if $tagTextFile =~ /DS_Store/;

    # Create a $name string that doesn't have .txt at the end so we can use with the other functions
    my $name = $tagTextFile;
    $name =~ s/.txt//g;


    # Open HEADERFILE that shares the same tag $name as our unicode text file
    # We're going to take the 64 bytes of the header file and add it to the new tag we're creating so it's a proper Myth tag
    open HEADERFILE, "$tagType-headers" . "/" . "$name.header"
      or die("Fatal Error:\tFile not found for verifying tag size:\t[$name.header]\n$!");
    binmode HEADERFILE;
    

    # This is without a doubt not the best way to do things. Getting the size of the header
    while ( defined( $tagLine = <HEADERFILE> ) ) {
        chomp $tagLine;
        $tagSize = length $tagLine;
    }

    # Error Check: Is the header file 64 bytes? If not kill the application for debugging
    if ($tagSize != 64) {
	die("Fatal Error:\tThe header file: [$name.header] is not 64 bytes. It should be.\n");
    }

    # Open the file, read in 64 bytes and assign to $tagHeader so it can be written to the new file
    open HEADERFILE, "$tagType-headers" . "/" . "$name.header"
      or die("Fatal Error:\tFile for storing in tagHeader:\t[$name.header]\n$!");
    binmode HEADERFILE;

    # assign $name.header first 64 bytes to $tagHeader
    $tagrv = read( HEADERFILE, $tagbuffer, 64 )
      or die "Fatal Error:\tCouldn't read from file:\t[$name.header]\n$!";
    $tagHeader = unpack( "H*", $tagbuffer );


    # Open NEWTAG for writing in the $newTagsFolder and give it the $name of the tag
    open NEWTAG, ">", "$newTagsFolder" . "/" . "$name";
    binmode NEWTAG;

    # Write the $tagHeader to the NEWTAG
    print NEWTAG pack( "H*", $tagHeader );


    # Open the CHARMAPFILE and create the necessary hash for converting characters from it
    open CHARMAPFILE, "<:encoding(UTF16-LE)", "$charfile";

    # Read the CHARMAPFILE as one string and then break it into an array of individual characters
    my $string = <CHARMAPFILE>;
    my @list = unpack( '(a)*', $string );

    # The first 32 (0-31) characters are reserved so we start at 32 for assigning the charmap slots
    my $i = 32;
    foreach (@list) {
        my $char = $_;
        my $ordi = ord($char);
        $charmap{$char} = $i;
	
	# Uncomment the following line to possibly show what the character <-> slot relation will be. Although this probably will not work in your console unless it supports Unicode
	# print "Character:\t" . $char . "\tHex:\t".unpack('H*',$_)."\tOrdinal:\t" . $ordi . "\tArray:\t".$i."\n";

        $i++;
    }

    # Finally, we open the TRANSLATORTEXT $tagTextFile so we can translate it with the charmap hash
    open TRANSLATORTEXT, "<:encoding(UTF16-LE)", "$uniDirName" . "/" . "$tagTextFile"
      or die("Fatal Error:\tCouldn't open TRANSLATORTEXT:\t$tagTextFile\n$!");

    # Going through each line of the file..
    while (<TRANSLATORTEXT>) {
        my $string = $_;

	# Each character is put into an array, individually
        my @list = unpack( 'a' x length($string), $string );

	# For each character of the current line in @list...
        foreach (@list) {

            my $char = $_;
            my $ordi = ord($char);
	    
	    # Move on to the next character in @list if we see feff, as that is the identifier from Unicode files we don't want
	    next if (sprintf( '%2.2x', unpack( 'U0U*', $char ) )  eq 'feff');

            binmode STDOUT, ":utf8";

	    # If the $charmap{$char} has a match, print its value to NEWTAG in HEX form
	    # elsif the character is 0A (newline) print it as 0D (mythline) - warning.. not the correct technical terms
	    # elsif the character is < 32 AND if it DOES NOT EQUAL 0D print the characters hex value exactly as they are 
	    # Characters < 32 stay the same between Myth and Unicode files, which is why we print them exactly as they are. I think.
	    # else the character has no match, print a warning that the character does not exist in the charmap and thusly did not translate properly

            if ( $charmap{$char} ) {
                print NEWTAG pack( "H*", sprintf( "%X", $charmap{$char} ) );
            }
            elsif ( sprintf( '%2.2x', unpack( 'U0U*', $char ) ) eq "0a" ) {

                print NEWTAG pack( "H*", '0D' );

            }
            elsif ( hex( sprintf( '%2.2x', unpack( 'U0U*', $char ) ) ) < 32 ) {

                if ( sprintf( '%2.2x', unpack( 'U0U*', $char ) ) ne "0d" ) {
                    print NEWTAG pack( "H*", sprintf( '%2.2x', unpack( 'U0U*', $char ) ) );
                }
            }
			elsif ( sprintf( '%2.2x', unpack( 'U0U*', $char ) ) eq "feff" ) {
				#FEFF identifies a text file
				#This space purely to ignore it.
            }
            else {
                print "Character '" . sprintf( '%2.2x', unpack( 'U0U*', $char ) ) . "' does not exist in charmap.\n";
            }
        }

    }

}   