#!C:\perl\bin\perl.exe -w
use warnings;
use strict;
use feature 'unicode_strings';
use utf8;
use File::Basename;
use Encode qw/encode decode/;
use feature qw(switch say);

my $buffer;
my $file;
my $size;
my $i;

#read $dirName from command arguments
my ($dirName) = @ARGV;

#basic error checking
my $countArgs = @ARGV;

if ( !$dirName ) { die("Error: You forgot to specify a tag directory.\n"); }

if ($countArgs != 1) {
die (
     "Error:\tYou provided the wrong number of arguments.\n\n" .
     "Syntax: perl japanese2unicode.pl \"TAGDIRECTORY\"\n\n".
     "\t\"TAGDIRECTORY\" Valid Choices (Must be in quotes):\n" .
     "\t\t\"dialog string lists\"\n\t\t\"string lists\"\n\t\t\"text\"\n\t\t\"templates\"\n"
    );
}

given ($dirName) {
    when ("dialog string lists") { break; }
    when ("string lists")        { break; }
    when ("text")                { break; }
    when ("templates")           { break; }
    default {
        die("Error: Invalid Tag Type\n\nValid Choices (Must be in quotes):\n\t\"dialog string lists\"\n\t\"string lists\"\n\t\"text\"\n\t\"templates\"\n"
        );
    }
}

#Create the directories that we'll be writing our headers and unicode text files to
mkdir("$dirName-headers");
mkdir("$dirName-unicode");

#Open the directory of tags we're going to be converting
opendir( TAGDIR, $dirName ) or die "Error: Can't opendir $dirName.\n$!";

#Cycle through each file name in the opened TAGDIR
while ( defined( $file = readdir(TAGDIR) ) ) {

	# skip . and .. # ...
    # skip DS_Store (mac file)
	next if $file =~ /^\.\.?$/;    
    next if $file =~ /DS_Store/;

    print "Converting $file...\n";
	
	# open the original tag $file
    open ORIGINALTAGFILE, "$dirName" . "/" . "$file"
      or die("Fatal Error:\tUnable to open file:\t[$file]\n$!");
    binmode ORIGINALTAGFILE;

	# determine total $size of the file for use later
    $size = -s "$dirName/$file";

	# open the tag file and read the first 64 bytes
    open ORIGINALTAGFILE, "$dirName" . "/" . "$file"
      or die("Fatal Error: Failed to read tag file for header information:\t$file\n$!");
    binmode ORIGINALTAGFILE;
    my $rv = read( ORIGINALTAGFILE, $buffer, 64 )
      or die("Fatal Error:\tFailed to read 64 bytes to buffer for file:\t[$file]\n");

	# Assign the first 64 bytes to $tagHeader
    my $tagHeader = unpack( "H*", $buffer );
	
	# Read the remaining bytes from the file and assign to $lines
	# If there's an error here at line 80 it means the tag file wasn't longer than 64 bytes which is cause for issue
    $rv = read( ORIGINALTAGFILE, $buffer, $size - 64 )
      or die "Fatal Error:\tCan't find anything beyond the 64 bytes header for file:\t[$file]\n";
	  
    my $lines = unpack( "a*", $buffer );

	# Extract the $name of the file without extension, the tag name
    my $name = basename($file);
	
	# Create a file $name.header in the $dirName-headers with the 64 bytes from $tagHeader
    open OUTPUT, ">:raw", "$dirName" . '-headers/' . "$name.header"
      or die "Fatal Error:\tCould not create header file:\t[$name.header]\n$!";
	print OUTPUT pack( "H*", $tagHeader );
	
	# Create our new unicode text file and write the FEFF identifier and cp932 decoded to utf8 $lines to it
    open UNICODEFILE, ">:raw:encoding(UTF16-LE):utf8",
      "$dirName" . '-unicode/' . "$file.txt"
      or die("Fatal Error:\tCould not create unicode text file:\t[$file.txt]\n$!");
    print UNICODEFILE "\x{FEFF}";

	# decode our cp932 $lines into $newlines
	# Replace all 0D(\r) lines with 0D0A(\r\n) so that they show up right in text editors
	my $newlines = decode( "cp932", $lines );
	$newlines =~ s/\r/\r\n/g;

	# Print our $newlines with the fixed line feeds
	print UNICODEFILE $newlines;

	#print UNICODEFILE decode( "cp932", $lines );

	
	#Let's be honest now, nothing really makes sure the conversion completed nicely.
}
closedir(TAGDIR);
