#!/usr/local/ActivePerl-5.14/bin/perl -w
use feature 'unicode_strings';
use utf8;
use warnings;
use strict;
use File::Basename;
use Encode qw/encode decode/;
use feature qw(switch say);

my ($basedirname) = @ARGV;
my $tagLine;
my $tagSize;
my $tagrv;
my $tagHeader;
my $tagbuffer;
my $textFile;

#basic error checking
my $countArgs = @ARGV;

if ( !$basedirname ) { die("Error: You forgot to specify a tag directory.\n"); }

if ($countArgs != 1) {
die (
     "Error:\tYou provided the wrong number of arguments.\n\n" .
     "Syntax: perl shift2myth.pl \"TAGDIRECTORY\"\n\n".
     "\t\"TAGDIRECTORY\" Valid Choices (Must be in quotes):\n" .
     "\t\t\"dialog string lists\"\n\t\t\"string lists\"\n\t\t\"text\"\n\t\t\"templates\"\n"
    );
}

given ($basedirname) {
    when ("dialog string lists") { break; }
    when ("string lists")        { break; }
    when ("text")                { break; }
    when ("templates")           { break; }
    default {
        die("Fatal Error: Invalid Tag Type\n\nValid Choices:\n\t\"dialog string lists\"\n\t\"string lists\"\n\t\"text\"\n\t\"templates\"\n"
        );
    }
}

#Create a $unicodeTags string of the folder name that will store our new tags
#Create a $uniDirName string of the folder that should have our text files to convert
my $unicodeTags = $basedirname . "-unicode_tags";
my $uniDirName  = $basedirname . "-unicode";

#Create directory with $unicodeTags name for storing our new tag files that we're about to create
mkdir("$unicodeTags");

#Open the $basedirname-unicode folder to get a list of all text files ready to convert
opendir( UNICODEDIR, $uniDirName )
  or die "Fatal Error:\tUnable to open the directory \"$uniDirName\"\n$!";

#Read the file names in the opened UNICODEDIR one file name at a time
while ( defined( $textFile = readdir(UNICODEDIR) ) ) {

    # skip . and .. # ...
    # skip DS_Store if you're on the mac
    next if $textFile =~ /^\.\.?$/;
    next if $textFile =~ /DS_Store/;

#prepare $name for dealing with files throughout the script
#remove .txt from the end of the filename so we can use $name when looking up header and unicode files.
    my $name = $textFile;
    $name =~ s/.txt//g;

    printf "Converting %s to a tag file...\n", $name;

#open $name's header file so we can get the first 64 bytes that should go into our NEW tag
    open HEADERFILE, "$basedirname-headers" . "/" . "$name.header"
      or die("Fatal Error:\tUnable to open the header file:\t[$name.header]\n$!");

    #I wonder what would happen if I removed this line? Hmm...
    binmode HEADERFILE;

    #read the first 64 bytes from the .header file
    $tagrv = read( HEADERFILE, $tagbuffer, 64 )
      or die "Fatal Error:\tCouldn't read from HEADERFILE:\t[$name.header]\n";

    #assign those first 64 bytes in hexadecimal form to $tagHeader
    $tagHeader = unpack( "H*", $tagbuffer );

#open a new tag file in $unicodeTags (tagtype-unicode_tags) named our tag $name for writing
    open NEWTAG, ">", "$unicodeTags" . "/" . "$name";

    #I wonder what would happen if I removed this line? Hmm...
    binmode NEWTAG;

    #write the $tagHeader to the new tag file
    print NEWTAG pack( "H*", $tagHeader );

    #open the translated unicode text file so it can be converted
    open TRANSLATORTEXT, "<:encoding(UTF16-LE)",
      "$uniDirName" . "/" . "$textFile"
      or die("Fatal Error:\tCouldn't open TRANSLATORTEXT:\t[$textFile]\n$!");

#parse the translated text file and write to new tag file encoded in cp932
#assign the $_ to something more human readable
#remove the beginning FEFF unicode identifier from the string
#make the $string cp932 format, at this point it looks like it should in the tag file
#search the $string and remove all the \n lines put in that jack things up. Otherwise known as "0A".
    while (<TRANSLATORTEXT>) {
        my $string = $_;
		$string =~ s/^\x{FEFF}//g;
		$string = encode( "cp932", $string );
        $string =~ s/\n//g;
		
		#writing the cp932 encoded $string to the new tag file
        print NEWTAG $string;
    }
	
#nothing really checks to make sure conversion is complete
}
